import os,re,sys,random,string,time
import montylingua
__version__ = "2.0"
__author__ = "hugo@media.mit.edu"
__url__ = 'www.conceptnet.org'
class GeneratePredicates:


    omcsraw = './omcsraw.txt'
    preds_file = './generated_predicates.txt'
    residue_file = './residue.txt'
    autocorrect_file = './autocorrect.txt'
    second_order = [re.compile('The statement ".*?" is true because (.*)'),
                    re.compile('Another way to say ".*?" is "(.*)"'),
                    re.compile('spatial concept: (.*)'),
                    re.compile('To understand the event ".*?", it is important to know that (.*)'),
                    re.compile('picture description: (.*)')
                    ]
    patterns = [
        [re.compile('^(.+) would make you want to (.+)'),
         lambda x,y: [('DesirousEffectOf',x,y)]
         ],
        [re.compile('^The effect of (.+) is that (.+)'),
         lambda x,y: [('EffectOf',x,y)]
         ],
        [re.compile('^The effect of (.+) is (.+)'),
         lambda x,y: [('EffectOf',x,y)]
         ],
        [re.compile('^The consequence of (.+) is that (.+)'),
         lambda x,y: [('EffectOf',x,y)]
         ],
        [re.compile('^The consequence of (.+) is (.+)'),
         lambda x,y: [('EffectOf',x,y)]
         ],
        [re.compile('^Something that might happen as a consequence of (.+) is that (.+)'),
         lambda x,y: [('EffectOf',x,y)]
         ],
        [re.compile('^Sometimes (.+) causes (.+)'),
         lambda x,y: [('EffectOf',x,y)]
         ],
        [re.compile('^One of the things you do when you (.+) is (.+)'),
         lambda x,y: [('SubeventOf',x,y)]
         ],
        [re.compile('^Something that might happen when you (.+) is (.+)'),
         lambda x,y: [('SubeventOf',x,y)]
         ],
        [re.compile('^Something you might do while (.+) is (.+)'),
         lambda x,y: [('SubeventOf',x,y)]
         ],
        [re.compile('^Something that might happen while (.+) is (.+)'),
         lambda x,y: [('SubeventOf',x,y)]
         ],
        [re.compile('^When you (.+) you do the following: 1[.] (.*?), 2[.] (.*?), 3[.] (.*?), 4[.] (.*?), 5[.] (.*)'),
         lambda x,a,b,c,d,e: [('SubeventOf',x,j) for j in [a,b,c,d,e]]
         ],
        [re.compile('^The first thing you do when you (.+) is (.+)'),
         lambda x,y: [('FirstSubeventOf',x,y)]
         ],
        [re.compile('^The last thing you do when you (.+) is (.+)'),
         lambda x,y: [('LastSubeventOf',x,y)]
         ],
        [re.compile('^Something you need to do before you (.+) is (.+)'),
         lambda x,y: [('PrerequisiteEventOf',x,y)]
         ],
        [re.compile('^(.+) is part of (.+)'),
         lambda x,y: [('PartOf',x,y)]
         ],
        [re.compile('^An? ([^ ]+) has ([^t][^o].+)'),
         lambda x,y: [('PartOf',x,y)]
         ],
        [re.compile('^(.+) (can be|may be|are|is) (typically |usually |sometimes |often |actually |generally |probably |mostly |only )?made (out )?(of |from )(.+)'),
         lambda x,t1,t2,t3,t4,y: [('MadeOf',x,y)]
         ],
        [re.compile('^(.+) (are|is) (a|an|some) (kind|type|sort|instance|example) of (.+)'),
         lambda x,t1,t2,t3,y: [('IsA',x,y)]
         ],
        [re.compile('^(A|An|a|an|the|The) (.+) is (a|an) (.+)'),
         lambda t1,x,t2,y: [('IsA',x,y)]
         ],
        [re.compile('^([^ ]+( [^ ]+)) is (a|an) (.+)'),
         lambda x,t1,t2,y: [('IsA',x,y)]
         ],
        [re.compile('^Something you (might )?find (.+) is (.+)'),
         lambda t,x,y: [('LocationOf',y,x)]
         ],
        [re.compile('^Somewhere (.+) can be is (.+)'),
         lambda x,y: [('LocationOf',x,y)]
         ],
        [re.compile('^(Someone|someone|Something|something) can be ((around|close to|in|at|on|near|between|before|in front of|under|beside|next to|through|off|upon|across|against|by) .+)'),
         lambda x,y,t: [('LocationOf',x,y)]
         ],
        [re.compile('^You are likely to find (.+) ((around|close to|in|at|on|near|between|before|in front of|under|beside|next to|through|off|upon|across|against|by) .+)'),
         lambda x,y,t: [('LocationOf',x,y)]
         ],
        [re.compile('^(.+) (is|are|can be|may be) used (for|to) (.+)'),
         lambda x,t1,t2,y: [('UsedFor',x,y)]
         ],
        [re.compile('^(.+) (is|are) for (.+)'),
         lambda x,t,y: [('UsedFor',x,y)]
         ],
        [re.compile('^You can use (.+) (for|to) (.+)'),
         lambda x,t,y: [('UsedFor',x,y)]
         ],
        [re.compile('^An activity (.+) can do is (.+)'),
         lambda x,y: [('CapableOf',x,y)]
         ],
        [re.compile('^You would (.+) because you want to (.+)'),
         lambda x,y: [('MotivationOf',x,y)]
         ],
        [re.compile('^You would (.+) because you want (.+)'),
         lambda x,y: [('MotivationOf',x,y)]
         ],
        [re.compile('^You would (.+) because you (.+)'),
         lambda x,y: [('MotivationOf',x,y)]
         ],
        [re.compile('^You would (.+) because (.+)'),
         lambda x,y: [('MotivationOf',x,y)]
         ],
        [re.compile('^If you want to (.+) then you should (.+)'),
         lambda x,y: [('MotivationOf',y,x)]
         ],
        [re.compile('^If you want to (.+) you should (.+)'),
         lambda x,y: [('MotivationOf',y,x)]
         ],
        [re.compile('^If you want to (.+), (.+)'),
         lambda x,y: [('MotivationOf',y,x)]
         ],
        [re.compile('^(A|An|a|an) (.+) wants (to )?(.+)'),
         lambda t1,x,t2,y: [('DesireOf',x,y)]
         ],
        [re.compile('^(A|An|a|an) (.+) (does not|doesn\'t) want to (.+)'),
         lambda t1,x,t2,y: [('DesireOf',x,'not '+y)]
         ],
        [re.compile('^(A|An|a|an) (.+) (does not|doesn\'t) want (.+)'),
         lambda t1,x,t2,y: [('DesireOf',x,'no '+y)]
         ],
        [re.compile('Things that are often found together are: (.+?), (.+?), (.+?), (.+?), (.+)'),
         lambda x1,x2,x3,x4,x5: [('ThematicKLine',i,j) for i in [x1,x2,x3,x4,x5] for j in filter(lambda z:z!=i,[x1,x2,x3,x4,x5] )]
         ],
        [re.compile('Things that are often found together are: (.+?), (.+?), (.+?), (.+)'),
         lambda x1,x2,x3,x4: [('ThematicKLine',i,j) for i in [x1,x2,x3,x4] for j in filter(lambda z:z!=i,[x1,x2,x3,x4] )]
         ],
        [re.compile('Things that are often found together are: (.+?), (.+?), (.+)'),
         lambda x1,x2,x3: [('ThematicKLine',i,j) for i in [x1,x2,x3] for j in filter(lambda z:z!=i,[x1,x2,x3] )]
         ],
        [re.compile('^([^ ]+( [^ ]+)?) is the (.+)'),
         lambda x,t1,y: [('DefinedAs',x,y)]
         ],
        [re.compile('^([^ ]+( [^ ]+)?) are (.+)'),
         lambda x,t1,y: [('IsA',x,y)],
         "lambda x,t1,y: self.constrain_pos(x,'^(NN |NNS |NNP |NNPS |JJ |JJR |JJS )*(NNS|NNP|NNPS)$') and self.constrain_chunktype(y,'NX')"
         ],
        [re.compile('^((a|an|the|A|An|The|some|Some|most|Most|every|Every|usually|Usually|typically|Typically) [^ ]+( [^ ]+)?) is (.+)'),
         lambda x,t1,t2,y: [('PropertyOf',x,y)],
         "lambda x,t1,t2,y: self.constrain_chunktype(x,'NX') and self.constrain_chunktype(y,'AX')"
         ],
        [re.compile('^([^ ]+( [^ ]+)?) are (.+)'),
         lambda x,t1,y: [('PropertyOf',x,y)],
         "lambda x,t1,y: self.constrain_pos(x,'^(NN |NNS |NNP |NNPS |JJ |JJR |JJS )*(NNS|NNP|NNPS)$') and self.constrain_chunktype(y,'AX')"
         ],
        [re.compile('^([^ ]+( [^ ]+)?) is ([^ ]+( [^ ]+)?)$'),
         lambda x,t1,y,t2: [('PropertyOf',x,y)],
         "lambda x,t1,y,t2: self.constrain_pos(x,'^(NN |NNS |NNP |NNPS |JJ |JJR |JJS )?(NN|NNP|NNPS)$') and self.constrain_chunktype(y,'AX')"
         ],
        [re.compile('^((a |an |the |A |An |The |some |Some |most |Most |every |Every |usually |Usually |typically |Typically |).+) (can )?(have |has )(.+)'),
         lambda x,t1,t2,t3,y: [('PartOf',x,y)],
         "lambda x,t1,t2,t3,y: self.constrain_chunktype(x,'NX') and self.constrain_chunktype(y,'NX')"
         ],
        [re.compile('^((a |an |the |A |An |The |some |Some |most |Most |every |Every |usually |Usually |typically |Typically |).+) (can be|may be|could be|should be|is|are) (.+)'),
         lambda x,t1,t2,y: [('CapableOfReceivingAction',x,y)],
         "lambda x,t1,t2,y: self.constrain_chunktype(x,'NX') and self.constrain_chunktype(y,'VP')"
         ],
        [re.compile('^((a|an|the|A|An|The|some|Some|most|Most|every|Every|usually|Usually|typically|Typically) [^ ]+) (can )?(.+)'),
         lambda x,t1,t2,y: [('CapableOf',x,y)],
         "lambda x,t1,t2,y: self.constrain_chunktype(x,'NX') and self.constrain_chunktype(y,'VP_noncopula')"
         ],
        [re.compile('^((a|an|the|A|An|The|some|Some|most|Most|every|Every|usually|Usually|typically|Typically) [^ ]+( [^ ]+)) (can )?(.+)'),
         lambda x,t1,t2,t3,y: [('CapableOf',x,y)],
         "lambda x,t1,t2,t3,y: self.constrain_chunktype(x,'NX') and self.constrain_chunktype(y,'VP_noncopula')"
         ],
        [re.compile('^([^ ]+) (can )?(.+)'),
         lambda x,t1,y: [('CapableOf',x,y)],
         "lambda x,t1,y: self.constrain_pos(x,'^(NN |NNS |NNP |NNPS |JJ |JJR |JJS )*(NNS|NNPS|NN|NNP)$') and self.constrain_chunktype(y,'VP_noncopula')"
         ],
        [re.compile('^([^ ]+( [^ ]+)) (can )?(.+)'),
         lambda x,t1,t2,y: [('CapableOf',x,y)],
         "lambda x,t1,t2,y: self.constrain_pos(x,'^(NN |NNS |NNP |NNPS |JJ |JJR |JJS )*(NNS|NNPS|NN|NNP)$') and self.constrain_chunktype(y,'VP_noncopula')"
         ],
        ]
    def constrain_chunktype(self,text,chunktype):

        chunked = self.chunk(text)
        if chunktype in ['AX','NX']:
            chunked_toks = chunked.split()
            if len(chunked_toks)>2 and chunked_toks[0]=='('+chunktype and chunked_toks[-1]==chunktype+')':
                return 1
            else:
                return 0
        elif chunktype in ['VP','VP_noncopula','VP_copula']:
            chunked_toks = chunked.split()
            if len(chunked_toks)>2 and chunked_toks[0]=='(VX' and chunked_toks[-1] in ['VX)','NX)']:
                if chunktype == 'VP':
                    return 1
                elif chunktype == 'VP_noncopula':
                    if len(chunked_toks[1].split('/'))==2 and chunked_toks[1].split('/')[0].lower() not in ['is','be','being','been','are','were','was','am']:
                        return 1
                    else:
                        return 0
                elif chunktype == 'VP_copula':
                    if len(chunked_toks[1].split('/'))==2 and chunked_toks[1].split('/')[0].lower() in ['is','be','being','been','are','were','was','am']:
                        return 1
                    else:
                        return 0
        elif chunktype == 'PX':
            if len(chunked_toks)>3:
                m = re.search('^(IN |TO ){1,2}\(NX (.+) NX\)$',chunked)
                if m:
                    return 1
            else:
                return 0
        return 0
    def constrain_pos(self,text,pos_regexp):

        tagged = self.tag(text)
        tags = ' '.join(map(lambda x:x.split('/')[1],tagged.split()))
        cre = re.compile(pos_regexp)
        m = cre.search(tags)
        if m:
            return 1
        return 0
    def process_sentence(self,text):

        for pattern in self.patterns:
            if len(pattern)==2:
                regexp,func = pattern
                additional_constraints = None
            elif len(pattern)==3:
                regexp,func,additional_constraints = pattern
            m = regexp.search(text)
            if m:
                if additional_constraints:
                    res = apply(eval(additional_constraints,locals()),list(m.groups()))
                    if not res:
                        continue
                preds = apply(func,list(m.groups()))
                preds = map(lambda pred: map(lambda x:x.replace('"','\''),pred),preds)
                return preds
        extraction = self.generate_extraction(text)
        nxs = self.jist_nxs(extraction)
        pairs = [(x,y) for x in nxs for y in filter(lambda a:a!=x,nxs)]
        shallow_preds = map(lambda x:('ConceptuallyRelatedTo',x[0],x[1]),pairs)
        return shallow_preds
    def jist_nxs(self,extraction):

        infos = extraction
        entities = []
        for info in infos:
            nxs = info['noun_phrases']
            entities += nxs
        return entities
    def generate_extraction(self,text):

        self.m=self.theMontyLingua
        sentences = self.m.split_sentences(text)
        tokenized = map(self.m.tokenize,sentences)
        tagged = map(self.m.tag_tokenized,tokenized)
        chunked = map(self.m.chunk_tagged,tagged)
        extracted = map(self.m.extract_info,chunked)
        return extracted
    def __init__(self):

        self.theMontyLingua = montylingua.MontyLingua.MontyLingua()
        self.autocorrect_kb = {}
        items = filter(lambda line:line.strip()!='',open(self.autocorrect_file,'r').read().split('\n'))
        map(lambda x:self.autocorrect_kb.__setitem__(x.split()[0],' '.join(x.split()[1:])),items)
    def output_chunked_omcs(self,omcsraw="./omcsraw.txt"):

        output_filename='./omcsraw_chunked.txt'
        g=open(output_filename,'w')
        f=open(omcsraw,'r')
        cur_line = f.readline()
        while cur_line:
            chunked = self.chunk_and_lemmatise(cur_line)
            g.write(chunked+'\n')
            cur_line = f.readline()
        f.close()
        g.close()
        return "Outputted OMCS Chunked to: "+output_filename+'!'
    def chunk_and_lemmatise(self,text):

        toked_text = self.theMontyLingua.tokenize(text)
        tagged_text = self.theMontyLingua.tag_tokenized(toked_text)
        lemmatised_text = self.theMontyLingua.lemmatise_tagged(tagged_text)
        chunked = self.theMontyLingua.chunk_lemmatised(lemmatised_text)
        return chunked
    def chunk(self,text):

        toked_text = self.theMontyLingua.tokenize(text)
        tagged_text = self.theMontyLingua.tag_tokenized(toked_text)
        chunked = self.theMontyLingua.chunk_tagged(tagged_text)
        return chunked
    def tag(self,text):

        toked_text = self.theMontyLingua.tokenize(text)
        tagged_text = self.theMontyLingua.tag_tokenized(toked_text)
        return tagged_text
    def autocorrect(self,text):

        toks = text.split()
        toks = map(lambda x:self.autocorrect_kb.get(x,x),toks)
        return ' '.join(toks)
    def pp_predicate(self,pred):

        linktype = pred[0]
        args = pred[1:]
        return '('+linktype+' '+' '.join(map(lambda x:'"'+x+'"',args))+')'
    def generate_predicates(self):

        g = open(self.preds_file,'w')
        h = open(self.residue_file,'w')
        f = open(self.omcsraw,'r')
        cur_line = f.readline()
        while cur_line:
            cur_line = cur_line.strip()
            cur_line = cur_line.strip('.')
            cur_line = self.autocorrect(cur_line)
            if not cur_line:
                cur_line = f.readline()
                continue
            for second_order_wrapper in self.second_order:
                m = second_order_wrapper.search(cur_line)
                if m:
                    cur_line = m.groups()[0]
            preds = self.process_sentence(cur_line)
            if preds:
                g.write('\n'.join(map(lambda x: self.pp_predicate(x),preds))+'\n')
                if preds[0][0]=='ConceptuallyRelatedTo':
                    h.write(cur_line+'\n')
            else:
                h.write(cur_line+'\n')
            cur_line = f.readline()
        f.close()
        g.close()
        h.close()
if __name__ == '__main__':
    g = GeneratePredicates()
    g.generate_predicates()
    print g.chunk_and_lemmatise("Something that might happen when you score a home run")