import xmlrpclib

__version__="1.0"
__author__ = "jhe@media.mit.edu"
__url__ = 'web.media.mit.edu/~jhe/GoalNet'

class GoalNetClient:
    def __init__(self):
        self.server=xmlrpclib.ServerProxy("http://localhost:8001")

    def find_path(self,initialNode,goalNode):
        """Find the most plausible path between the initial state and
        the goal state.

        INPUT: Two string representing the initial and goal state.
        OUTPUT: An array with thxfe path."""
        return self.server.find_path(initialNode,
                                     goalNode)

    def find_next_state(self,currentState):
        """Find the actions that are likely to happen next.

        INPUT: An array with string representing the set of current states.
        OUTPUT: An array of the tupes of possible next states and score."""
        return self.server.find_next_state(currentState)

    def find_temporal_related_state(self,currentState):
        """Assumes that GoalNet is a undirected graph and find the actions
        that are temporally related to it. It is the combination of
        GoalNet.fine_next_state and GoalNet.find_previous_state functions.

        INPUT: An array with string representing the set of current states.
        OUTPUT: An array of the tupes of possible next states and score."""

        return self.server.find_temporal_related_state(currentState)

    def find_previous_state(self,currentState):
        """Find the actions that are likely to happen  before.

        INPUT: An array with string representing the set of current states.
        OUTPUT: An array of the tupes of possible next states and score."""

        return self.server.find_previous_state(currentState)
    
if __name__=='__main__':
     gnc=GoalNetClient()

     print "Find path"
     answer=gnc.find_path("I hunger","I eat hamburger")
     print answer
     
     print "Find next state"
     answer=gnc.find_next_state(["I see movie"])

     for i in answer:
         print i
         
     print "Find temporal related states"
     answer=gnc.find_temporal_related_state(["I see movie"])

     for i in answer:
         print i

     print "Find previous state"
     answer=gnc.find_previous_state(["I see movie"])

     for i in answer:
         print i
