"""
Module GoalNet - An every day life planer
"""
import ConceptNetDB
import LifeNet,GoalNet
import GoalNetSpreading

import DocXMLRPCServer

__version__="1.0"
__author__ = "jhe@media.mit.edu"
__url__ = 'web.media.mit.edu/~jhe/GoalNet'

class GoalNetServer:
    def __init__(self):
        cn=ConceptNetDB.ConceptNetDB()
        self.ln=LifeNet.LifeNet(cn)

    def __new_goalnetspreading__(self):
        gn=GoalNet.GoalNet(self.ln)
        return GoalNetSpreading.GoalNetSpreading(gn)
    
    def find_path(self,initialNode,goalNode):
        """Find the most plausible path between the initial state and
        the goal state.

        INPUT: Two string representing the initial and goal state.
        OUTPUT: An array with thxfe path."""

        gns=self.__new_goalnetspreading__()
        answer=gns.find_path([initialNode],
                             [goalNode],
                             "all_nodes",
                             "all_nodes",
                             100)
        answer=map(lambda x:x[1],answer)

        return answer

    def find_next_state(self,currentState):
        """Find the actions that are likely to happen next.

        INPUT: An array with string representing the set of current states.
        OUTPUT: An array of the tupes of possible next states and score."""

        gns=self.__new_goalnetspreading__()
        answer=gns.find_next_state(currentState,
                                   "all_nodes",
                                   "all_nodes")
        answer=map(lambda x:x[1],answer)

        return answer

    def find_temporal_related_state(self,currentState):
        """Assumes that GoalNet is a undirected graph and find the actions
        that are temporally related to it. It is the combination of
        GoalNet.fine_next_state and GoalNet.find_previous_state functions.
        
        INPUT: An array with string representing the set of current states.
        OUTPUT: An array of the tupes of possible next states and score."""
        
        gns=self.__new_goalnetspreading__()
        answer=gns.find_temporal_related_state(currentState,
                                   "all_nodes",
                                   "all_nodes")
        answer=map(lambda x:x[1],answer)

        return answer

    def find_previous_state(self,currentState):
        """Find the actions that are likely to happen  before.

        INPUT: An array with string representing the set of current states.
        OUTPUT: An array of the tupes of possible next states and score."""

        gns=self.__new_goalnetspreading__()

        answer=gns.find_previous_state(currentState,
                                   "all_nodes",
                                   "all_nodes")
        answer=map(lambda x:x[1],answer)

        return answer

if __name__=='__main__':
    server=GoalNetServer()

    xmlrpc=DocXMLRPCServer.DocXMLRPCServer(('',8001))
    xmlrpc.register_introspection_functions()
    xmlrpc.register_instance(server)
    print "Starting: XML-RPC server"
    xmlrpc.serve_forever()
    print "XML-RPC server started"
    
    
    

