import sys
import re

import GoalNet,LifeNet
import ConceptNetDB

import GoalNetHeuristic

from Dijkstra import shortestPath

prog=re.compile("^[a-z]\w*[a-z0-9]$",re.I)
string_heuristic=[x for x in dir(GoalNetHeuristic) if prog.search(x) is not None]

UnknownStrategy=GoalNetHeuristic.__UnknownStrategy

__version__="1.0"
__author__ = "jhe@media.mit.edu"
__url__ = 'web.media.mit.edu/~jhe/GoalNet'

class GoalNetSpreading:
    initialEnergy=100.0
    averageEnergy=100.0

    def __init__(self,gn):
        self.goalnet=gn
        self.lifenet=gn.lifenet
        self.conceptnet=gn.lifenet.ConceptNet
        
        self.initialState=[]
        self.goalState=[]

        self.initialStateContext={}
        self.goalStateContext={}
        
        self.energy={}
        self.new_energy={}
        self.spreadParents={}
        self.spreadChildren={}
        self.lenParents=0
        self.lenChildren=0

        self.nodesActivated={}
        self.nodesExecutables={}
        self.time=1

    def set_initial_state(self,sentences,activation_strategy,gluenet_match=0):
        print "Setting initial state",sentences
        self.initialState=[]
        cn_nodes=[]
        for sentence in sentences:
            if gluenet_match:
                nodes_ids=self.lifenet.get_best_match(sentence)
            else:
                nodes_ids=[(1.0,self.lifenet.get_node_id(sentence))]
            for (plaus,node_id) in nodes_ids:
                print "Initializating initial state:",\
                      self.lifenet.get_node_string(node_id)
                self.initialState.append(node_id)
                self.add_energy(node_id,self.initialEnergy)
                self.spreadChildren[node_id]=["Initial state"]
                self.set_active(node_id,activation_strategy)
                cn_nodes.extend(list(self.lifenet.get_conceptnet_nodes(node_id)))

##        self.initialStateContext=self.get_context_score(cn_nodes)
        self.initialStateContext=cn_nodes
        self.lenChildren=len(self.spreadChildren)
        return self.initialState

    def set_goal_state(self,sentences,gluenet_match=0):
        print "Setting goal state",sentences
        self.goalState=[]
        cn_nodes=[]
        for sentence in sentences:
            if gluenet_match:
                nodes_ids=self.lifenet.get_best_match(sentence)
            else:
                nodes_ids = [(1.0,self.lifenet.get_node_id(sentence))]
            for (plaus,node_id) in nodes_ids:
                print "Initialization goal state:",\
                      self.lifenet.get_node_string(node_id)
                self.goalState.append(node_id)
                self.add_energy(node_id,self.initialEnergy)
                self.spreadParents[node_id]=["Goal state"]
                cn_nodes.extend(list(self.lifenet.get_conceptnet_nodes(node_id)))

##        self.goalStateContext=self.get_context_score(cn_nodes)
        self.goalStateContext=cn_nodes
        self.lenParents=len(self.spreadParents)
        return self.goalState

    def add_energy(self,node_id,value):
        if self.new_energy.has_key(node_id):
            self.new_energy[node_id]=self.new_energy[node_id]+value
        else:
            self.new_energy[node_id]=self.energy.get(node_id,0)+value

    def get_energy(self,node_id):
        ## CHECK: I will like to have [node_id] insted of get(node_id)
        return self.energy.get(node_id,0)

    def commit_energy(self):
        self.energy.update(self.new_energy)

        total_energy=sum(self.energy.itervalues())
        len_energy=len(self.energy)
        for (key,value) in self.energy.iteritems():
            self.energy[key]=value*self.averageEnergy*len_energy/total_energy

        self.new_energy={}

    def get_parents(self,node,limit_strategy):
        if hasattr(GoalNetHeuristic,limit_strategy):
            eval_string="GoalNetHeuristic."+limit_strategy+\
                     "(node,self.goalnet.get_parents,self,self.initialStateContext)"

            output=eval(eval_string)
            return output
        else:
            raise UnknownStrategy(limit_strategy)
    
    def get_children(self,node,limit_strategy):
        if hasattr(GoalNetHeuristic,limit_strategy):
            eval_string="GoalNetHeuristic."+limit_strategy+\
                         "(node,self.goalnet.get_children,self,self.goalStateContext)"

            output=eval(eval_string)
            return output
        else:
            raise UnknownStrategy(limit_strategy)
    
    def set_active(self,node_id,activation_strategy):
        executables={}
        self.nodesActivated[node_id]=self.time
        self.new_energy[node_id]=-10.0
        self.time+=1
        for (node,plaus) in self.get_children(node_id,
                                              limit_strategy=activation_strategy):
            ex_causes=self.nodesExecutables.get(node,[])
            ex_causes.append(node_id)
            self.nodesExecutables[node]=ex_causes
            executables[node]=self.get_energy(node)

        ## FIXME: might erase the energy of the initial state
        ##if self.energy.has_key(node_id):
        ##    del self.energy[node_id]

        return executables
        
    def spread(self,spreading_strategy):
        print "Spreading",len(self.spreadChildren),len(self.spreadParents)
        self.spread_children(spreading_strategy)
        self.spread_parents(spreading_strategy)
        self.commit_energy()


    def spread_children(self,spreading_strategy):
        for child in self.spreadChildren.keys():
            nodes=self.get_children(child,limit_strategy=spreading_strategy)
            energy=self.get_energy(child)
            for (node,plaus) in nodes:
                self.add_energy(node,energy*plaus)
                ## foo
                if self.spreadChildren.has_key(node):
                    self.spreadChildren[node].append(child)
                else:
                    self.spreadChildren[node]=[child]
                
    def spread_parents(self,spreading_strategy):
        for parent in self.spreadParents.keys():
            nodes=self.get_parents(parent,limit_strategy=spreading_strategy)
            energy=self.get_energy(parent)
            for (node,plaus) in nodes:
                self.add_energy(node,energy*plaus)
                #foo
                if self.spreadParents.has_key(node):
                    self.spreadParents[node].append(parent)
                else:
                    self.spreadParents[node]=[parent]
                    

    def activate(self,activation_strategy,activation_threshold,max_activation=1000):
        ## Relevant_nodes: All the nodes that has been spread by parents
        ## and childs that are executable
        ## Think how to speed it up!!!

        relevant_nodes={}
        tmp={}
##         for i in self.spreadParents.iterkeys():
##             if self.spreadChildren.has_key(i) and \
##                self.nodesExecutables.has_key(i) and \
##                self.get_energy(i) >= activation_threshold and \
##                not self.nodesActivated.has_key(i):
##                 relevant_nodes[i]=self.get_energy(i)
        for i in self.spreadParents.iterkeys():
            if self.spreadChildren.has_key(i) and \
               self.nodesExecutables.has_key(i):
                tmp[i]=self.get_energy(i)

        for i in tmp.iterkeys():
            if self.nodesExecutables.has_key(i) and \
                   self.get_energy(i) >= activation_threshold and \
                   not self.nodesActivated.has_key(i):
                relevant_nodes[i]=self.get_energy(i)
            
        nodes_activated=0
        while 1:
            if len(relevant_nodes)==0:
                break
            print "*** In active",len(self.spreadChildren),len(self.spreadParents),len(relevant_nodes)
            
            new_node=max(zip(relevant_nodes.itervalues(),relevant_nodes.iterkeys()))
            (energy,new_active_node)=new_node
            del relevant_nodes[new_active_node]
                        
            new_executables=self.set_active(new_active_node,activation_strategy)
            nodes_activated+=1
            
            print "\tActivating",self.lifenet.get_node_string(new_active_node),energy
            if self.are_done():
                break
            if max_activation<=nodes_activated:
                break
            
            for i in new_executables.iterkeys():
                if self.spreadParents.has_key(i) and \
                       self.spreadChildren.has_key(i) and \
                       self.get_energy(i) >= activation_threshold and \
                       not self.nodesActivated.has_key(i):
                    relevant_nodes[i]=self.get_energy(i)
        
    def are_done(self):
        return reduce(lambda x,y: self.nodesActivated.has_key(y) \
                      and x,self.goalState,True)

    def find_path(self,myinitialState,mygoalState, \
                  spreading_strategy,activation_strategy,activation_threshold,
                  gluenet_match=0):
        self.set_initial_state(myinitialState,activation_strategy,
                               gluenet_match=gluenet_match)
        self.set_goal_state(mygoalState,
                            gluenet_match=gluenet_match)

        if len(self.initialState)==0 or len(self.goalState)==0:
            return []

        num_cycles=0
        self.commit_energy()
        while not self.are_done():
            self.spread(spreading_strategy)
            self.activate(activation_strategy,activation_threshold)
            self.control_search(activation_strategy)
            if len(self.nodesActivated) > 10:
                print "Not node found by activation, using shorter path"
                break
            num_cycles=num_cycles+1
            if num_cycles>10:
                print "Can find an answer, using shorter path"
                break

        if self.are_done():
            Activated=map(lambda x:(x[1],self.lifenet.get_node_string(x[0])),
                          self.nodesActivated.iteritems())
            Activated.sort()
            return Activated
        else:
            return self.calculate_shortest_path(myinitialState,mygoalState)

    def control_search(self,activation_strategy):
        foo=len(self.spreadParents)

        if(self.lenParents==foo):
            self.expand_parents()
        else:
            self.lenParents=foo

        foo=len(self.spreadChildren)
        if(self.lenChildren==foo):
            self.expand_children(activation_strategy)
        else:
            self.lenChildren=foo

    def expand_parents(self):
        for node_id in self.spreadParents.iterkeys():
            self.goalnet.expand_by_node(node_id)

    def expand_children(self,activation_strategy):
        for node_id in self.spreadChildren.iterkeys():
            self.goalnet.expand_by_node(node_id)
            for (node,plaus) in self.get_children(node_id,
                                                  limit_strategy=activation_strategy):
                ex_causes=self.nodesExecutables.get(node,[])
                ex_causes.append(node_id)
                self.nodesExecutables[node]=ex_causes
    
    def calculate_shortest_path(self,myinitialState,mygoalState):
            try:
                path=shortestPath(self.goalnet.get_graph(),
                                  self.lifenet.get_best_string(myinitialState[0],
                                                               num_match=1)[0],
                                  self.lifenet.get_best_string(mygoalState[0],
                                                               num_match=1)[0])
            except KeyError:
                path=[]
                print "Upps!! Shorter path not work eather"

            output=[]
            for i in range(len(path)):
                output.append((i,path[i]))
            return output

    def get_context_score(self,cn_nodes):
        decode_node=self.conceptnet.decode_node
        nodeuid2zipped=self.conceptnet.nodeuid2zipped
        encode_node=self.conceptnet.encode_node
        zipped2nodeuid=self.conceptnet.zipped2nodeuid

        nodes_words=map(lambda x:(decode_node(nodeuid2zipped(x[0])),x[1]),
                        cn_nodes)
        
        context=self.conceptnet.get_context(nodes_words,
                                    textnode_list_weighted_p=1)
        context=map(lambda x:(zipped2nodeuid(encode_node(x[0])),x[1]),
                    context)
        return context

    def find_next_state(self,currentState, \
                        spreading_strategy,activation_strategy,
                        gluenet_match=0):

        self.set_initial_state(currentState,activation_strategy,
                               gluenet_match=gluenet_match)
        self.commit_energy()
        self.spread_children(spreading_strategy)
        self.commit_energy()
        Answer=[]
        for i in self.spreadChildren.iterkeys():
            Answer.append((self.get_energy(i),self.lifenet.get_node_string(i)))

        Answer.sort()
        Answer.reverse()
        return Answer

    def find_temporal_related_state(self,currentState, \
                                    spreading_strategy,activation_strategy,
                                    gluenet_match=0):
        self.set_initial_state(currentState,activation_strategy,
                               gluenet_match=gluenet_match)
        self.set_goal_state(currentState,
                            gluenet_match=gluenet_match)
        self.commit_energy()
        self.spread(spreading_strategy)
        Answer=[]
        for i in self.energy.iterkeys():
            Answer.append((self.get_energy(i),self.lifenet.get_node_string(i)))

        Answer.sort()
        Answer.reverse()
        return Answer

    def find_previous_state(self,currentState, \
                            spreading_strategy,activation_strategy,
                            gluenet_match=0):

        self.set_goal_state(currentState,gluenet_match=gluenet_match)
        self.commit_energy()
        self.spread_parents(spreading_strategy)
        self.commit_energy()
        Answer=[]
        for i in self.spreadParents.iterkeys():
            Answer.append((self.get_energy(i),self.lifenet.get_node_string(i)))

        Answer.sort()
        Answer.reverse()
        return Answer

    def run(self,init=["I get money"],goal=["I eat a hamburger"],\
            spreading_strategy='all_nodes',activation_strategy='all_nodes',\
            activation_threshold=0):

        if activation_threshold is None:
            activation_threshold=self.averageEnergy

        answer=self.find_path(init,goal,\
                              spreading_strategy,activation_strategy,activation_threshold)

##        print "Path find"
##        for i in answer:
##            print i

        return answer

if __name__=='__main__':
    cn=ConceptNetDB.ConceptNetDB()
    ln=LifeNet.LifeNet(cn)
    gn=GoalNet.GoalNet(gn)
    gns=GoalNetSpreading(gn)
    gns.find_next_state(["I have money"],"all_nodes","all_nodes")
