import re

import ConceptNetDB

from stopwords import is_stop_word

__version__="1.0"
__author__ = "jhe@media.mit.edu"
__url__ = 'web.media.mit.edu/~jhe/GoalNet'

class LifeNet:
    def __init__(self,cn):
        self.ConceptNet=cn

        self.dictNodes={} ## Nodes string to its node_id 
        self.indexNodes=[] ## Nodes node_id to its string 

        self.dictWords={} ## Words string in each node to its word_id
        self.indexWords=[] ## Nodes word_id to its string

        self.words_to_nodes=[] ## Index for words to nodes that contain them
        
        self.LifeNet_to_ConceptNet=[]
        ##Index for LifeNet nodes to ConcpetNet nodes
        self.ConceptNet_to_LifeNet={}
        ## Index for ConceptNet nodes to LifeNet nodes
        
        self.indexParents=[] ## Nodes' parents
        self.indexChildren=[] ## Nodes' childrens

        self.Graph={} ## A graph for closest path
        
        self.load()

    def load(self):
        self.__load_nodes()
        self.__load_graph()

    def __load_nodes(self):
        dictWords=self.dictWords
        indexWords=self.indexWords
        words_to_nodes=self.words_to_nodes
        def add_word(word_string,node_id):
            word_id=dictWords.get(word_string,None)
            if word_id is None:
                word_id=len(dictWords)
                dictWords[word_string]=word_id
                indexWords.append(word_string)
                words_to_nodes.append([node_id])
            else:
                self.words_to_nodes[word_id].append(node_id)

            return word_id
        ## End add_word
        
        dictNodes=self.dictNodes
        indexNodes=self.indexNodes
        indexParents=self.indexParents
        indexChildren=self.indexChildren
        LifeNet_to_ConceptNet=self.LifeNet_to_ConceptNet
        def add_node(node_string):
            node_id=dictNodes.get(node_string,None)
            if node_id is None:
                node_id=len(dictNodes)
                dictNodes[node_string]=node_id
                indexNodes.append(node_string)
                indexParents.append([])
                indexChildren.append([])
                LifeNet_to_ConceptNet.append([])

                map(lambda x:add_word(x,node_id),node_string.split())

            return node_id
        ## End add_node 

        ConceptNet_to_LifeNet=self.ConceptNet_to_LifeNet
        encode_node = self.ConceptNet.encode_node
        zipped2nodeuid = self.ConceptNet.zipped2nodeuid
        def link_conceptnet(text_node,plausability,node_id):
            encoded_node = encode_node(text_node)
            cn_id=zipped2nodeuid(encoded_node)
            LifeNet_to_ConceptNet[node_id].append((cn_id,plausability))

            arr_temp=ConceptNet_to_LifeNet.get(cn_id,[])
            arr_temp.append((node_id,plausability))
            ConceptNet_to_LifeNet[cn_id]=arr_temp
        ## End link_conceptnet
            
        f=open("lifenet-situations.lisp")

        p1  =re.compile("\((\d+) \"(.+)\" \(cn(.*)\) \)")
        p1_1=re.compile("([0|1].\d+) \"(.+?)\"")

        line=f.readline()
        num_nodes=0
        while line:
            m1=p1.match(line)
            if m1 is None:
                print "ERROR: Nodes",line
            else:
                ##node_key=int(m1.group(1))
                node_string=m1.group(2)
                node_id=add_node(node_string)
                map(lambda x:link_conceptnet(x[1],float(x[0]),node_id),\
                    p1_1.findall(m1.group(3)))
                
            line=f.readline()
            num_nodes+=1
            if num_nodes%1000==0:
                print "Loading node", num_nodes

        f.close()
        print "Nodes loaded!!"

    def __load_graph(self):
        prog=re.compile("\([=-~]> (0\.\d+) \"(.+)\" \"(.+)\" (\d+) (\d+)\)")

        f=open("lifenet-temporal.lisp")
        line=f.readline()

        num_lines=0
        dictNodes=self.dictNodes
        Graph=self.Graph
        while line: 
            result=prog.match(line)
            if result is not None:
                plausability =float(result.group(1))
                sentence_1   =result.group(2)
                sentence_2   =result.group(3)
                ##sentence_id_1=int(result.group(4))
                ##sentence_id_2=int(result.group(5))

                node_1_id=dictNodes[sentence_1]
                node_2_id=dictNodes[sentence_2]

                self.indexParents[node_2_id].append((node_1_id,plausability))
                self.indexChildren[node_1_id].append((node_2_id,plausability))

                my_dict=Graph.get(sentence_1,{})
                my_dict[sentence_2]=1/plausability
                Graph[sentence_1]=my_dict

                if not Graph.has_key(sentence_2):
                    Graph[sentence_2]={}
                ## This made the Graph unidirectional
                ##my_dict=Graph[sentence_2]
                ##my_dict[sentence_1]=1/plausability
                ##Graph[sentence_2]=my_dict
            else:
                print "ERROR graph:",line

            num_lines+=1
            if num_lines%1000==0:
                print "Loading edge",num_lines
            line=f.readline()

        f.close()
        print "GoalNet loaded!"

    def get_node_id(self,text):
        output=self.dictNodes.get(text,None)
        if output is None:
            match=self.get_best_match(text,num_match=1)
            if len(match)==0:
                output=None
            else:
                output=match[0][1]
        return output
        
    def get_node_string(self,n_id):
        if n_id is not None:
            return self.indexNodes[n_id]
        else:
            return None
        
    def get_word_id(self,text):
        return self.dictWords.get(text,None)

    def get_word_string(self,w_id):
        return self.indexWords[w_id]

    def get_wid_from_nid(self,n_id):
        node_string=self.get_node_string(n_id)
        return map(lambda x:self.get_word_id(x),node_string.split())
    
    def get_parents(self,id):
        return self.indexParents[id]

    def get_children(self,id):
        return self.indexChildren[id]

    def get_conceptnet_nodes(self,id):
        return self.LifeNet_to_ConceptNet[id]

    def get_nodesid_by_word(self,word):
        w_id=self.get_word_id(word)
        if w_id is not None:
            nodes=self.words_to_nodes[w_id]
        else:
            nodes=None
        return None
    
    def get_nodes_by_word(self,word):
        return map(lambda x:self.get_node_string(x),self.nodesid_by_word(word))

    def get_nodes_by_wordid(self,w_id):
        return self.words_to_nodes[w_id]

    def get_best_string(self,textstring,num_match=5):
        bestnodes=self.get_best_match(textstring,num_match=num_match)
        return map(lambda x:self.get_node_string(x[1]),
                   bestnodes)
    
    def get_best_match(self,textstring,num_match=5):
        string_filter=filter(lambda x:not is_stop_word(x),textstring.split())
        string_id=filter(lambda x: x is not None,
                         map(lambda x:self.get_word_id(x),string_filter))
        
        all_nodes=map(lambda x:self.get_nodes_by_wordid(x),string_id)

        get_wid_from_nid=self.get_wid_from_nid
        get_score=self.get_score
        unic_nodes={}
        for nodes in all_nodes:
            for node in nodes:
                if not unic_nodes.has_key(node):
                    words_id=get_wid_from_nid(node)
                    score=get_score(words_id,string_id)
                    unic_nodes[node]=score

        nodes_scored=zip(unic_nodes.itervalues(),unic_nodes.iterkeys())
        nodes_scored.sort()
        nodes_scored.reverse()
        return nodes_scored[:num_match]


    def get_score(self,node_1,node_2):
        return len(filter(lambda x:x in node_1,node_2))

    def get_nodes_by_cnid(self,n_id):
        return self.ConceptNet_to_LifeNet[n_id]

    def get_cnid_by_nodes(self,c_id):
        return self.LifeNet_to_ConceptNet[c_id]

